;(function(root, factory) {
  if (typeof define === 'function' && define.amd) {
    define([], factory);
  } else if (typeof exports === 'object') {
    module.exports = factory();
  } else {
    root.AboxWs = factory();
  }
}(this, function() {
'use strict';
var aboxWs = (function () {
	var videoSizeSocketURL = {
			default: 'wss://127.0.0.1:31341',
			fallback: 'ws://127.0.0.1:31340'
		},
		playerSocketURL = {
			default: 'wss://127.0.0.1:31351',
			fallback: 'ws://127.0.0.1:31350'
		};

	function startNewPlayer (url) {
		var playerSocket = null,
			videoIsPlaying = false,
			callbackQueue = [],
			open = false;
		try {
			playerSocket = new WebSocket(url);
		} catch (err) {
			console.error(err);
			playerSocket = null;
			return;
		}
		playerSocket.onerror = function() {
			if (url !== playerSocketURL.fallback) {
				startNewPlayer(playerSocketURL.fallback);
				return;
			}
			var c = callbackQueue.shift();
			if (typeof c != 'undefined' && typeof c.error != 'undefined' && c.error != null) {
				c.error('Playback fail, no connection with socket')
			}
		};
		playerSocket.onopen = function() {
			open = true;
		};
		playerSocket.onclose = function() {
			// Auto-reconnect
			if (open) {
				startNewPlayer(url);
			}
		}
		playerSocket.onmessage = function(e) {
			if (e.data === 'play start') {
				videoIsPlaying = true;
				var c = callbackQueue[0];
				if (typeof c != 'undefined' && typeof c.start != 'undefined' && c.start != null) {
					c.start();
				}
			} else if (e.data == 'play fail') {
				videoIsPlaying = false;
				var c = callbackQueue.shift();
				if (typeof c != 'undefined' && typeof c.error != 'undefined' && c.error != null) {
					c.error('Error while attempting to play video');
				}
			} else if (e.data === 'play stop') {
				videoIsPlaying = false;
				var c = callbackQueue.shift();
				if (typeof c != 'undefined' && typeof c.stop != 'undefined' && c.stop != null) {
					c.stop();
				}
			}
		};
		return {
			play: function(url, vol, startCb, stopCb, errorCb) {
				if (videoIsPlaying) {
					while (callbackQueue.length) {
						var c = callbackQueue.shift();
						if (typeof c != 'undefined' && typeof c.stop != 'undefined' && c.stop != null) {
							c.stop();
						}
					}
				}
				var callbacks = {
					start: startCb,
					stop: stopCb,
					error: errorCb
				};
				callbackQueue.push(callbacks);
				try {
					playerSocket.send('play ' + url + ' ' + vol.toString());
				} catch (err) {
					console.error(err);
				}
			}
		}
	};
	function setVideoSize(x, y, w, h, url) {
		var videoSizeSocket;
		url = url || videoSizeSocketURL.default;
		try {
			videoSizeSocket = new WebSocket(url);
		} catch (err) {
			console.error(err);
			return;
		}
		videoSizeSocket.onerror = function() {
			if (url !== videoSizeSocketURL.fallback) {
				setVideoSize(x, y, w, h, videoSizeSocketURL.fallback);
			}
		};
		videoSizeSocket.onopen = function() {
			var cmd = 'vsize ' + x + ' ' + y + ' ' + w + ' ' + h;
			try {
				videoSizeSocket.send(cmd);
			} catch (err) {
				console.error(err);
			}
			videoSizeSocket.close();
		}
	};

	return {
		setVideoSize: setVideoSize,
		initPlayer: function() {
			return startNewPlayer(playerSocketURL.default);
		}
	}
}());

return aboxWs;
}));
